

import java.awt.ComponentOrientation;
import java.awt.Desktop;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import com.mindfusion.scheduling.AeroEffect;
import com.mindfusion.scheduling.Calendar;
import com.mindfusion.scheduling.GlassEffect;
import com.mindfusion.scheduling.GlassEffectType;
import com.mindfusion.scheduling.Theme;

public class ApplicationMenu extends JMenuBar
{
	private static FileTypeFilter PNG_FILTER = new FileTypeFilter(".png", "PNG files");
	private static FileTypeFilter JPEG_FILTER = new FileTypeFilter(".jpeg", "JPEG files");
	
	private LinkedHashMap<JMenu, List<MenuItems>> options;
	private ContactsForm contacts;
	private AppointmentsForm appointments;

	public ApplicationMenu()
	{
		generateMenuOptions();
		
		for (JMenu menu : options.keySet())
		{
			for (MenuItems item : options.get(menu))
			{
				JMenuItem currentItem = item.getItem();
				currentItem.addActionListener(x -> 
				{
					System.out.println(item + " clicked");
					addFunctionalityOf(item);
				});
				
				menu.add(currentItem);
			}
			
			this.add(menu);
		}
	}
	
	public ContactsForm getContactsForm() // may be null
	{
		return this.contacts;
	}
	
	public AppointmentsForm getAppointmentsForm()
	{
		return this.appointments;
	}
	
	private void generateMenuOptions()
	{
		this.options = new LinkedHashMap<JMenu, List<MenuItems>>();
		
		options.put(new JMenu("File"), Arrays.asList(
				MenuItems.clear, MenuItems.load, MenuItems.save, 
				MenuItems.export, MenuItems.exportVisible,
				MenuItems.print, MenuItems.printPreview, /*MenuItems.previewReport,*/
				MenuItems.exit)); 
		
		options.put(new JMenu("Edit"), Arrays.asList(
				MenuItems.undo, MenuItems.redo));
		
		options.put(new JMenu("Schedule"), Arrays.asList(
				MenuItems.contacts, MenuItems.appointments));
		
		options.put(new JMenu("Layout"), Arrays.asList(
				MenuItems.leftToRight, MenuItems.rightToLeft));
		
		options.put(new JMenu("Theme"), Arrays.asList(
				MenuItems.standard, MenuItems.windows2003, MenuItems.lila,
				MenuItems.silver, MenuItems.vista, MenuItems.black,
				MenuItems.dark, MenuItems.light)); 
		
		options.put(new JMenu("Effects"), Arrays.asList(
				MenuItems.type1, MenuItems.type2, MenuItems.type3,
				MenuItems.type4, MenuItems.aero)); 
		
		options.put(new JMenu("Help"), Arrays.asList(
				MenuItems.about));
	}
	
	private void addFunctionalityOf(MenuItems item) 
	{
		Calendar calendar = Application.getInstance().getCalendar();
		GlassEffect effect = new GlassEffect();
		
		switch(item)
		{
			case clear:
				calendar.getSchedule().clear(); 
				break;
			case load: 
				Application.getInstance().load();
				break;
			case save:
				Application.getInstance().save();
				break;
			case export:
				export(calendar, false);
				break;
			case exportVisible: 
				export(calendar, true);
				break;
			case printPreview: 
				calendar.printPreview();
				break;
			case print: 
				calendar.print();
				break;
			case previewReport: //to-do
				//previewReport(calendar);
				break;
			case exit: 
				System.exit(0);
				break;
			
			case undo:
				calendar.getSchedule().undo();
				break;
			case redo:
				calendar.getSchedule().redo();
				break;
			
			case contacts: 
				this.contacts = new ContactsForm("Contacts");
				break;
			case appointments:
				appointments = new AppointmentsForm("Appointments");
				break;
			
			case leftToRight: 
				calendar.setRightToLeft(false);
				calendar.applyComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
				calendar.invalidate();
				break;
			case rightToLeft:
				calendar.setRightToLeft(true);
				calendar.applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT); // to-do : fix the graphics problem
				calendar.invalidate();
				break;
			
			case standard: 
				calendar.applyTheme(Theme.getStandard());
				break;
			case windows2003:
				calendar.applyTheme(Theme.getWindows2003());
				break;
			case lila: 
				calendar.applyTheme(Theme.getLila());
				break;
			case silver: 
				calendar.applyTheme(Theme.getSilver());
				break;
			case vista: 
				calendar.applyTheme(Theme.getVista());
				break;
			case black:
				calendar.applyTheme(Theme.getBlack());
				break;
			case dark:
				calendar.applyTheme(Theme.getDark());
				break;
			case light: 
				calendar.applyTheme(Theme.getLight());
				break;
			
			case type1: 
				effect.setType(GlassEffectType.Type1);
				calendar.getItemEffects().add(effect); 
				break;
			case type2:
				effect.setType(GlassEffectType.Type2);
				calendar.getItemEffects().add(effect); 
				break;
			case type3: 
				effect.setType(GlassEffectType.Type3);
				calendar.getItemEffects().add(effect); 
				break;
			case type4: 
				effect.setType(GlassEffectType.Type4);
				calendar.getItemEffects().add(effect); 
				break;
			case aero: 
				calendar.getItemEffects().add(new AeroEffect()); 
				break;
			
			case about: openWebsite();
				break;
		}
		
	}
	
	private void openWebsite() 
	{
		if (Desktop.isDesktopSupported() && Desktop.getDesktop().isSupported(Desktop.Action.BROWSE)) {
		    try {
				Desktop.getDesktop().browse(new URI("https://mindfusion.eu/java-scheduler.html"));
			}
		    catch (IOException | URISyntaxException e) 
		    {
				e.printStackTrace();
			}
		}
	}
	
	private void export(Calendar calendar, boolean visible) 
	{
		JFileChooser saveImage = new JFileChooser();
		saveImage.setDialogTitle("Export as image");
		saveImage.setAcceptAllFileFilterUsed(false);
		
		saveImage.setFileFilter(PNG_FILTER);
		saveImage.setFileFilter(JPEG_FILTER);
		
		int s = saveImage.showSaveDialog(null);
		
		if (s == JFileChooser.APPROVE_OPTION)
		{
			try 
			{
		        BufferedImage bufferedImage = calendar.createImage(visible);
				
				String fileName = saveImage.getSelectedFile().getAbsolutePath();
				
				if (saveImage.getFileFilter().equals(PNG_FILTER))
				{
					fileName += ".png";
					File pngFile = new File(fileName);
					ImageIO.write(bufferedImage, "png", pngFile);
				}
				else if (saveImage.getFileFilter().equals(JPEG_FILTER))
				{
					fileName += ".jpeg";
					File jpegFile = new File(fileName);
					ImageIO.write(bufferedImage, "jpeg", jpegFile);
				}
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		}
	}
	
	private void previewReport(Calendar calendar)
	{
		System.out.println("to-do");
		//to-do :
	}
}
